@@ -246,8 +246,17 @@ class ConvRNN2D(RNN):
         initial_state = K.sum(initial_state, axis=1)
         shape = list(self.cell.kernel_shape)
         shape[-1] = self.cell.filters
+
+        if K.backend() == 'tensorflow':
+            # We need to force this to be a tensor
+            # and not a variable, to avoid variable initialization
+            # issues.
+            import tensorflow as tf
+            kernel = tf.zeros(tuple(shape))
+        else:
+            kernel = K.zeros(tuple(shape))
         initial_state = self.cell.input_conv(initial_state,
-                                             K.zeros(tuple(shape)),
+                                             kernel,
                                              padding=self.cell.padding)
         # Fix for Theano because it needs
         # K.int_shape to work in call() with initial_state.
@@ -491,8 +500,6 @@ class ConvLSTM2DCell(Layer):
             incompatible with specifying any `strides` value != 1.
         activation: Activation function to use
             (see [activations](../activations.md)).
-            If you don't specify anything, no activation is applied
-            (ie. "linear" activation: `a(x) = x`).
         recurrent_activation: Activation function to use
             for the recurrent step
             (see [activations](../activations.md)).
@@ -509,8 +516,8 @@ class ConvLSTM2DCell(Layer):
         unit_forget_bias: Boolean.
             If True, add 1 to the bias of the forget gate at initialization.
             Use in combination with `bias_initializer="zeros"`.
-            This is recommended in [Jozefowicz et al.]
-            (http://www.jmlr.org/proceedings/papers/v37/jozefowicz15.pdf).
+            This is recommended in [Jozefowicz et al. (2015)](
+            http://www.jmlr.org/proceedings/papers/v37/jozefowicz15.pdf).
         kernel_regularizer: Regularizer function applied to
             the `kernel` weights matrix
             (see [regularizer](../regularizers.md)).
@@ -597,7 +604,6 @@ class ConvLSTM2DCell(Layer):
         self._recurrent_dropout_mask = None
 
     def build(self, input_shape):
-
         if self.data_format == 'channels_first':
             channel_axis = 1
         else:
@@ -623,6 +629,7 @@ class ConvLSTM2DCell(Layer):
             constraint=self.recurrent_constraint)
         if self.use_bias:
             if self.unit_forget_bias:
+                @K.eager
                 def bias_initializer(_, *args, **kwargs):
                     return K.concatenate([
                         self.bias_initializer((self.filters,), *args, **kwargs),
@@ -817,8 +824,6 @@ class ConvLSTM2D(ConvRNN2D):
             incompatible with specifying any `strides` value != 1.
         activation: Activation function to use
             (see [activations](../activations.md)).
-            If you don't specify anything, no activation is applied
-            (ie. "linear" activation: `a(x) = x`).
         recurrent_activation: Activation function to use
             for the recurrent step
             (see [activations](../activations.md)).
@@ -835,8 +840,8 @@ class ConvLSTM2D(ConvRNN2D):
         unit_forget_bias: Boolean.
             If True, add 1 to the bias of the forget gate at initialization.
             Use in combination with `bias_initializer="zeros"`.
-            This is recommended in [Jozefowicz et al.]
-            (http://www.jmlr.org/proceedings/papers/v37/jozefowicz15.pdf).
+            This is recommended in [Jozefowicz et al. (2015)](
+            http://www.jmlr.org/proceedings/papers/v37/jozefowicz15.pdf).
         kernel_regularizer: Regularizer function applied to
             the `kernel` weights matrix
             (see [regularizer](../regularizers.md)).
@@ -901,9 +906,9 @@ class ConvLSTM2D(ConvRNN2D):
 
     # References
         - [Convolutional LSTM Network: A Machine Learning Approach for
-        Precipitation Nowcasting](http://arxiv.org/abs/1506.04214v1)
-        The current implementation does not include the feedback loop on the
-        cells output
+          Precipitation Nowcasting](http://arxiv.org/abs/1506.04214v1)
+          The current implementation does not include the feedback loop on the
+          cells output
     """
 
     @interfaces.legacy_convlstm2d_support
